/*--------------------------------------------------
ESP-Webserver Dateien im SPIFFS
v0.5 MSoft 18.05.2016
32MBit-Flash: 4M(1M SPIFFS), NodeMCU
--------------------------------------------------*/

#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
#include <FS.h>

const char* ssid = "xxxxxxx";
const char* password = "xxxxxxxxx";

void WiFiStart(void);
String getContentType(String filename);                    // setzt Content-Typ nach Dateiendung
void notFound(void);                                       // schickt 404-Meldung 

void handleUnknown(void);

void handleUpload(void);                                   // Datei-Upload HTML
void handleUploadStart(void);                              // Datei-Upload holt File

void handleIndex(void);
void handleForm(void);
void handleEingabe(void);
void handleSchalter(void);

// Create an instance of the webserver on Port 80
ESP8266WebServer webserver(80);

// ###############################################################

void setup()
{
  // start serial
  Serial.begin(115200);                           
  Serial.println("");
  Serial.println("RESET");
  
  // Initialize file system.
  if (!SPIFFS.begin())
  {
    Serial.println("SPIFFS nicht initialisiert!");   
    while(1)                                        // ohne SPIFFS geht es sowieso nicht...
    {
      yield();
    }
  }
  Serial.println("SPIFFS ok");
   
  // inital connect
  WiFi.mode(WIFI_STA);
  WiFiStart();
  IPAddress myAddr = WiFi.localIP();
  Serial.println(myAddr);

  // Start the server
  webserver.on("/", handleIndex);
  webserver.on("/index.html", handleIndex);
  webserver.on("/index.htm", handleIndex);
  webserver.on("/upload.html", HTTP_GET, handleUpload);
  webserver.on("/upload.html", HTTP_POST, [](){ webserver.send(200, "text/plain", ""); }, handleUploadStart);

  webserver.on("/form.html", handleForm);
  webserver.on("/eingabe.html", handleEingabe);
  webserver.on("/schalter.html", handleSchalter);
  
  webserver.onNotFound(handleUnknown);
  webserver.begin();

  Serial.println("HTTP server gestartet Port 80");
}

// ###############################################################

void loop()
{
  // check if WLAN is connected
  if (WiFi.status() != WL_CONNECTED)
  {
    WiFiStart();
  }
  webserver.handleClient();
}

// ###############################################################

//---------------------------------------------------------------

void handleUnknown()
{
  String filename = webserver.uri();
  Serial.print("URL: ");
  Serial.println(filename);

  File pageFile = SPIFFS.open(filename, "r"); 
  if (pageFile)
  {
    String contentTyp = getContentType(filename);
    Serial.print("Content-Typ: ");
    Serial.println(contentTyp);

    size_t sent = webserver.streamFile(pageFile, contentTyp);
    pageFile.close(); 
  }
  else
  {
    notFound(); 
  }
}

// ###############################################################

void notFound()
{
  String sHTML = F("<html><head><title>404 Not Found</title></head><body><h1>Not Found</h1><p>The requested URL was not found on this webserver.</p></body></html>");
  webserver.send(404, "text/html", sHTML);    
}

// ----------------------------------------------------------------------

String getContentType(String filename)
{
  if(webserver.hasArg("download"))
    return "application/octet-stream";
  else if(filename.endsWith(".ico"))
    return "image/x-icon";
  else if(filename.endsWith(".htm"))
    return "text/html";
  else if(filename.endsWith(".html"))
    return "text/html";
  else if(filename.endsWith(".css"))
    return "text/css";
  else if(filename.endsWith(".js"))
    return "application/javascript";
  else if(filename.endsWith(".png"))
    return "image/png";
  else if(filename.endsWith(".gif"))
    return "image/gif";
  else if(filename.endsWith(".jpg"))
    return "image/jpeg";
  else if(filename.endsWith(".ico"))
    return "image/x-icon";
  else if(filename.endsWith(".xml"))
    return "text/xml";
  else if(filename.endsWith(".pdf"))
    return "application/x-pdf";
  else if(filename.endsWith(".zip"))
    return "application/x-zip";
  else if(filename.endsWith(".gz"))
    return "application/x-gzip";
  return "text/plain";
}

// ----------------------------------------------------------------------

void WiFiStart()
{
  // Connect to WiFi network
  WiFi.begin(ssid, password);
  Serial.print("WiFi");
  while (WiFi.status() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(500);
  }
  Serial.println("connected");
}

//---------------------------------------------------------------
//             Webseiten
//---------------------------------------------------------------

void handleUpload()
{
  String filename = webserver.uri();
  File pageFile = SPIFFS.open(filename,"r"); 
  if (pageFile)
  {
    String sHTML = pageFile.readString();
    pageFile.close();  
    
    webserver.send(200, "text/html", sHTML);
  }
  else
  {
    notFound(); 
  }
}

// ----------------------------------------------------------------------

void handleUploadStart()
{
  Serial.println("Datei-Upload"); 

  static File fsUploadFile;                                         // aktueller Upload

  HTTPUpload& upload = webserver.upload();
  if(upload.status == UPLOAD_FILE_START)
  {
    String dateiname = upload.filename;
    if(!dateiname.startsWith("/"))
    {
      dateiname = "/" + dateiname;
    }
    
    Serial.print("handleFileUpload Name: ");
    Serial.println(dateiname);
    fsUploadFile = SPIFFS.open(dateiname, "w");
    dateiname = String();
  }
  else if(upload.status == UPLOAD_FILE_WRITE)
  {
    if(fsUploadFile)
    {
      fsUploadFile.write(upload.buf, upload.currentSize);
      Serial.println(upload.currentSize,DEC);
    }  
  }
  else if(upload.status == UPLOAD_FILE_END)
  {
    if(fsUploadFile)
    {
      fsUploadFile.close();
    }  
    Serial.print("handleFileUpload Size: ");
    Serial.println(upload.totalSize);
  }
    
//  webserver.send(200, "text/plain", "");
}

// ----------------------------------------------------------------------
 
void handleSchalter()
{
  String filename = webserver.uri();

  File pageFile = SPIFFS.open(filename, "r"); 
  if (pageFile)
  {
    String sHTML = pageFile.readString();
    pageFile.close();

    if (webserver.hasArg("dose1"))
    {
      if (webserver.arg("dose1") == "1") 
      {
//        tx_send(15);            
        Serial.println("Computer Ein");
      }
      else
      {
//        tx_send(14);            
        Serial.println("Computer Aus");
      }
    }  
    if (webserver.hasArg("dose2"))
    {
      if (webserver.arg("dose2") == "1")
      {
//        tx_send(7);            
        Serial.println("TV Ein");
      }
      else
      {
//        tx_send(6);            
        Serial.println("TV Aus");
      }
        
    }
    if (webserver.hasArg("dose3"))
    {
      if (webserver.arg("dose3") == "1")
      {
//        tx_send(11);            
        Serial.println("Lampe Ein");
      }
      else
      {
//        tx_send(10);            
        Serial.println("Lampe Aus");
      }
    }  
    if (webserver.hasArg("dose4"))
    {
      if (webserver.arg("dose4") == "1") 
      {
//        tx_send(3);            
        Serial.println("LED Ein");
      }
      else
      {
//        tx_send(2);            
        Serial.println("LED Aus");
      }
    }    
    webserver.send(200, "text/html", sHTML);
  }
  else
  {
    notFound(); 
  }
}

//---------------------------------------------------------------

void handleEingabe()
{ 
  String filename = webserver.uri();

  File pageFile = SPIFFS.open(filename, "r"); 
  if (pageFile)
  {
    String sHTML = pageFile.readString();
    pageFile.close();
    if (webserver.hasArg("display"))
    {
      sHTML.replace("~display~",webserver.arg("display"));
    }
    else
    {
      sHTML.replace("~display~","");
    }
    
    if (webserver.hasArg("speed"))
    {
      sHTML.replace("~speed~",webserver.arg("speed"));
    }
    else
    {
      sHTML.replace("~speed~","");
    }

    webserver.send(200, "text/html", sHTML);
  }
  else
  {
    notFound(); 
  }
}

//---------------------------------------------------------------

void handleForm()
{
  String filename = webserver.uri();

  File pageFile = SPIFFS.open(filename, "r"); 
  if (pageFile)
  {
    String sHTML = pageFile.readString();
    pageFile.close();
    
    String name;
    if (webserver.hasArg("vname"))
    {
      name = webserver.arg("vname"); 
    }
    if (webserver.hasArg("zname"))
    {
      name += " " + webserver.arg("zname");  
    }
    name.trim();
    if (name.length() > 0)
    {
      sHTML.replace("~name~","Sie hei&szlig;en " + name + ".<br>");
    }
    else
    {
      sHTML.replace("~name~","");
    }

    String zutaten;
    if (webserver.hasArg("zutat1"))
    {
      zutaten += " und " + webserver.arg("zutat1"); 
    }
    if (webserver.hasArg("zutat2"))
    {
      zutaten += " und " + webserver.arg("zutat2"); 
    }
    if (webserver.hasArg("zutat3"))
    {
      zutaten += " und " + webserver.arg("zutat3"); 
    }
     
    if (zutaten.startsWith(" und "))
    {
      String temp = zutaten.substring(5);
      sHTML.replace("~zutaten~","Sie haben " + temp + " gew&auml;hlt.<br>");
    }
    else
    {
      sHTML.replace("~zutaten~","");
    }
    
    if (webserver.hasArg("Zahlmethode"))
    {
      sHTML.replace("~zahlweise~","Sie wollen mit " + webserver.arg("Zahlmethode") + " bezahlen.");     
    }
    {
      sHTML.replace("~zahlweise~","");
    }
        
    webserver.send(200, "text/html", sHTML);
  }
  else
  {
    notFound(); 
  }
}

//---------------------------------------------------------------

void handleIndex()
{
  String filename = webserver.uri();
  if (filename == "/")
  {
    filename = "/index.html";
  }
  Serial.print("URL: ");
  Serial.println(filename);

  File pageFile = SPIFFS.open(filename, "r"); 
  if (pageFile)
  {
    String sHTML = pageFile.readString();
    pageFile.close();  

    Serial.println("Replace...");

    sHTML.replace("~t_bz~", "21,3");
    sHTML.replace("~f_bz~", "48");
    sHTML.replace("~t_ba~", "22,4");
    sHTML.replace("~f_ba~", "52");
    sHTML.replace("~t_wz~", "22,4");
    sHTML.replace("~f_wz~", "49");
    sHTML.replace("~t_ke~", "18,2");
    sHTML.replace("~f_ke~", "51");
    sHTML.replace("~t_sz~", "21,6");
    sHTML.replace("~t_ge~", "-21,8");

    sHTML.replace("~druck~", "1018");

    sHTML.replace("~e_nas~", "32,2");
    sHTML.replace("~e_kuehl~", "0,0");
    sHTML.replace("~e_tv~", "1,8");
    sHTML.replace("~e_bastel~", "7,1");
    sHTML.replace("~e_pc~", "92,4");
    sHTML.replace("~e_uhr~", "3,7");
       
    webserver.send(200, "text/html", sHTML);
  }
  else
  {
    notFound(); 
  }
}

